-- <!DOCTYPE ADA>
-- <ADA>

--------------------------------------------------------------------------------
-- <PROLOGUE>
-- <UNIT>        OSI_Files
--
-- <DESCRIPTION> This package provides routines to manipulate regular files
--               and directories.  
--
-- <HISTORY>
--
-- <ORIGIN>
-- <AUTHOR>      Gerardo A. Colon
-- <DATE>        11 July 1990
--
-- <COMPANY>     Science Applications International Corporation
-- <ADDRESS>     311 Park Place Boulevard, Suite 360
-- <ADDRESS>     Clearwater, Florida 34619
--
-- <REMARK>      Developed for the STARS program under task S40.
-- </ORIGIN>
--
-- <CHANGE>
-- <AUTHOR>      Gerardo A. Colon
-- <DATE>        August 21, 1990
-- <REMARK>      Added type File_Descriptor and the functions 
-- <REMARK>      Is_A_Terminal_Device and Get_Terminal_Device_Name.  
-- <REMARK>      Defined Standard_Input, Standard_Output, and 
-- <REMARK>      Standard_Error file descriptors.
-- </CHANGE>
--
-- </HISTORY>
--
-- <NOTES>       Dependencies =>
--                 ( Operating_System => None ,
--                   Compiler         => None ,
--                   Device           => None ) ;
--
-- </PROLOGUE>
--------------------------------------------------------------------------------

with OSI_Standard ;
use  OSI_Standard ;

pragma Elaborate ( OSI_Standard ) ;

pragma Page ;

--------------------------------------------------------------------------------
-- package OSI_Files (specification).
--------------------------------------------------------------------------------

package OSI_Files is

-- A file is identified with a filename.  The length of a filename can be upto 
-- a maximum of OSI_Standard.Portable_Filename_Limit.  A filename is represented
-- as an OSI_String defined in OSI_Standard.
--

  type Access_Modes is ( Read_OK , Write_OK , Execute_OK ) ;
--
-- This type represents the different access modes for a file.  The values
-- of this type are:
--
--   Read_OK      The file can be opened for input, or other read operations 
--                may be performed on the file by the caller.
--
--   Write_OK     The file can be opened for output, or other write operations
--                may be performed on the file by the caller.
--
--   Execute_OK   The file may be executed.
--
-- In order to perform operations on files, the user needs to have access 
-- to the file which allows to perform the desired operation on the file.
-- The user can query to see if he has certain access to a file through 
-- the procedure Accessible.
--

  type Access_Mode_Set is array ( Access_Modes ) of Boolean ;
--
-- This type is an array of Boolean values that indicates a desired Access mode.
--

  type File_Descriptor is range 0 .. Portable_Open_Files ;
--
-- This type is used to represent a file or device available for I/O operations
-- within the system.
--

  Standard_Input  : constant File_Descriptor := 0 ;
  Standard_Output : constant File_Descriptor := 1 ;
  Standard_Error  : constant File_Descriptor := 2 ;
--
-- Predefined values of file descriptors which represent files normally 
-- opened by the system before the process starts.
--

  procedure Create_Directory ( Directory_Name : in OSI_String ) ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Create_Directory
--
-- <DESCRIPTION> This procedure creates a directory with the name specified
--               by Directory_Name.
--
-- <EXCEPTIONS>  File_Exists               =>
--                 There is already an entry in the file system named by 
--                 the parameter Directory_Name.
--               Filename_Syntax_Error     =>
--                 Directory_Name contains one or more characters not 
--                 supported by the implementation for a filename.
--               Filename_Too_Long         =>
--                 The length of Directory_Name exceeds the maximum length.
--               No_Space_Left_On_Device   =>
--                 The file system does not contain enough space to hold the
--                 new directory or to extend the contents of the parent 
--                 directory of the new directory.
--               No_Such_File_Or_Directory =>
--                 A component of the path prefix of Directory_Name does not
--                 exist, or Directory_Name is null.
--               Not_A_Directory           =>
--                 A component of the path prefix named in Directory_Name is
--                 not a directory.
--               Permission_Denied         =>
--                 One or more components of the parameter Directory_Name does
--                 not have Search permission, or write permission is denied 
--                 on the parent directory of the name given by Directory_Name.
--               Read_Only_File_System     =>
--                 The parent directory of the file named by Directory_Name
--                 resides on a read-only file system.
--               Too_Many_Links            =>
--                 The link count in the parent directory of the directory 
--                 specified by Directory_Name exceeds the systems maximum.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  procedure Unlink ( File_Name : in OSI_String ) ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Unlink
--
-- <DESCRIPTION> This procedure removes the link named by the parameter
--               File_Name and decrements the link count of the file 
--               referenced by File_Name.  When the link count becomes zero,
--               and no process has the file open, the space occupied by the
--               file is freed and the file is no longer accessible.  This 
--               procedure operates only on regular files.
--
-- <EXCEPTIONS>  Filename_Syntax_Error =>
--                 File_Name contains one or more characters not 
--                 supported by the implementation for a filename.
--               Filename_Too_Long     =>
--                 The length of File_Name exceeds its maximum.
--               Is_A_Directory        =>
--                 File_Name identifies a directory.
--               Not_A_Directory       =>
--                 A component of the path prefix named by file File_Name is
--                 not a directory.
--               Permission_Denied     =>
--                 Search permission is denied to one or more of the components 
--                 of the path prefix of File_Name or Write permission is
--                 denied on the parent directory of the file named by 
--                 File_Name.
--               Read_Only_File_System =>
--                 The parent directory of the file named by Directory_Name
--                 resides on a read-only file system.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  procedure Remove_Directory ( File_Name : in OSI_String ) ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Remove_Directory
--
-- <DESCRIPTION> This procedure removes the directory named by the parameter
--               File_Name.  The directory will be removed only if it is empty.
--               The effect of removing a directory is that the link named by 
--               File_Name is removed, and the link count of the directory is
--               decremented.  When the link count becomes zero, and no process
--               has the directory open, the space occupied by the directory is
--               freed and the directory is no longer accessible.
--
-- <EXCEPTIONS>  Directory_Not_Empty   =>
--                 The directory named by Filename is not empty.
--               Filename_Syntax_Error =>
--                 File_Name contains one or more characters not 
--                 supported by the implementation for a filename.
--               Filename_Too_Long     =>
--                 The length of File_Name exceeds its maximum.
--               Not_A_Directory       =>
--                 A component of the path given by File_Name does not identify 
--                 a directory or does not exist.
--               Permission_Denied     =>
--                 Search permission is denied to one or more of the components 
--                 of the path prefix of File_Name or Write permission is
--                 denied on the parent directory of the directory named by 
--                 File_Name.
--               Read_Only_File_System =>
--                 The parent directory of the directory named by File_Name
--                 resides on a read-only file system.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  procedure Rename ( File_Name     : in OSI_String ;
                     New_File_Name : in OSI_String ) ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Rename
--
-- <DESCRIPTION> This procedure changes the name of the file specified by
--               File_Name to the name specified by New_File_Name.
--
-- <EXCEPTIONS>  Directory_Not_Empty       =>
--                 The file named by New_File_Name is an existing directory,
--                 and that directory is not empty.
--               Filename_Syntax_Error     =>
--                 File_Name or New_File_Name contain one or more characters 
--                 not supported by the implementation for a filename.
--               Filename_Too_Long         =>
--                 The length of File_Name or New_File_Name exceed its 
--                 maximum.
--               Invalid_Argument          =>
--                 File_Name is a directory and the file named by parameter 
--                 New_File_Name contains a path prefix that names the same 
--                 directory as parameter File_Name.
--               Is_A_Directory            =>
--                 The file named by New_File_Name is a directory and the file 
--                 named by File_Name is not a directory.
--               No_Space_Left_On_Device   =>
--                 The file system does not contain enough space to hold 
--                 the new file name by parameter New_File_Name or to extend
--                 the contents of the parent directory of New_File_Name.
--               No_Such_File_Or_Directory =>
--                 The file named by File_Name does not exist in the file 
--                 system or a component of the path prefix of New_File_Name
--                 does not exist.
--               Not_A_Directory           =>
--                 A component of the path prefix of File_Name is not a 
--                 directory.
--               Permission_Denied         =>
--                 Search permission is denied on any of the components of the 
--                 path prefix of either File_Name or New_File_Name or Write 
--                 permission is denied on the parent directory of either 
--                 File_Name or New_File_Name.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  procedure Copy_File ( File_Name     : in OSI_String ;
                        New_File_Name : in OSI_String ) ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Copy_File
--
-- <DESCRIPTION> This procedure copies the contents of the regular file 
--               identified by File_Name to the file identified by 
--               New_File_Name.  If New_File_Name identifies an existing
--               file, the file will be overwritten.
--
-- <EXCEPTIONS>  Filename_Syntax_Error     =>
--                 File_Name or New_File_Name contain one or more characters 
--                 not supported by the implementation for a filename.
--               Filename_Too_Long         =>
--                 The length of File_Name or New_File_Name exceed its maximum.
--               Is_A_Directory            =>
--                 File_Name or New_File_Name identify a directory.
--               No_Space_Left_On_Device   =>
--                 The file system does not contain enough space to hold 
--                 the new file named by parameter New_File_Name or to extend
--                 the contents of the parent directory of New_File_Name.
--               No_Such_File_Or_Directory =>
--                 The file named by File_Name does not exist in the file 
--                 system or a component of the path prefix of New_File_Name
--                 does not exist.
--               Not_A_Directory           =>
--                 A component of the path prefix of File_Name is not a 
--                 directory.
--               Permission_Denied         =>
--                 Search permission is denied on any of the components of the
--                 path prefix of either File_Name or New_File_Name; or Write
--                 permission is denied on the parent directory of the file 
--                 named by New_File_Name.
--               Too_Many_Links            =>
--                 The link count of the parent directory of parameter 
--                 New_File_Name would exceed its maximum.
--               Too_Many_Open_Files           =>
--                 The number of open files in the process exceeds the 
--                 implementations defined limit.
--               Too_Many_Open_Files_In_System =>
--                 The number of open files in the system exceeds the 
--                 implementations defined limit.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  function Is_File ( File_Name : in OSI_String ) return Boolean ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Is_File
--
-- <DESCRIPTION> This function returns True if the file specified by File_Name
--               is a regular file.  Otherwise, the function returns False.
--
-- <EXCEPTIONS>  Filename_Syntax_Error     =>
--                 File_Name contains one or more characters not 
--                 supported by the implementation for a filename.
--               Filename_Too_Long         =>
--                 The length of File_Name exceeds the maximum length.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  function Is_Directory ( File_Name : in OSI_String ) return Boolean ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Is_Directory
--
-- <DESCRIPTION> This function returns True is the file specified by File_Name
--               is a directory.  Otherwise, the function returns False.
--
-- <EXCEPTIONS>  Filename_Syntax_Error     =>
--                 Directory_Name contains one or more characters not 
--                 supported by the implementation for a filename.
--               Filename_Too_Long         =>
--                 The length of File_Name exceeds the maximum length.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  generic
    with procedure Action ( File_Name : in OSI_String ) ;
  procedure Directory_Reader ( Directory_Name : in OSI_String ) ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Directory_Reader
--
-- <DESCRIPTION> This procedure iterates through the files in the directory
--               specified by Directory_Name, calling the procedure Action 
--               for every file.
--
-- <EXCEPTIONS>  Filename_Syntax_Error         =>
--                 Directory_Name contains one or more characters not 
--                 supported by the implementation for a filename.
--               Filename_Too_Long             =>
--                 The length of Directory_Name exceeds its maximum.
--               No_Such_File_Or_Directory     =>
--                 The directory named by Directory_Name does not exist.
--               Not_A_Directory               =>
--                 A component of the parameter Directory_Name does not refer
--                 to a directory.
--               Permission_Denied             =>
--                 Search permission was denied for a component of parameter
--                 Directory_Name or read permission was denied on the 
--                 directory named by Directory_Name.
--               Too_Many_Open_Files           =>
--                 The number of open files in the process exceeds the 
--                 implementations defined limit.
--               Too_Many_Open_Files_In_System =>
--                 The number of open files in the system exceeds the 
--                 implementations defined limit.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  procedure Accessible ( File_Name   : in OSI_String ;
                         Permissions : in Access_Mode_Set ) ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Accessible
--
-- <DESCRIPTION> This procedure shall return without error if the file 
--               specified by File_Name supports the access modes indicated 
--               by Permissions.  
--
-- <EXCEPTIONS>  Filename_Syntax_Error     =>
--                 File_Name contains one or more characters not 
--                 supported by the implementation for a filename.
--               Filename_Too_Long         =>
--                 The length of parameter File_Name exceeds its maximum.
--               No_Such_File_Or_Directory =>
--                 The file named by parameter File_Name does not exist.
--               Not_A_Directory           =>
--                 A component of the path prefix of the parameter File_Name 
--                 is not a directory.
--               Permission_Denied         =>
--                 Search permission is denied for a component of parameter 
--                 File_Name or The file access specified by Permissions is
--                 is not allowed on the file named by File_Name.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  function Host_Name_Of ( File_Name : in OSI_String ) return String ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Host_Name_Of
--
-- <DESCRIPTION> This function returns the host operating system name of the 
--               given file (name suitable for the use by predefined Ada IO
--               packages).
--
-- <EXCEPTIONS>  Filename_Syntax_Error     =>
--                 File_Name contains one or more characters not 
--                 supported by the implementation for a filename.
--               Filename_Too_Long         =>
--                 The length of File_Name exceeds its maximum.
--               
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  function Is_A_Terminal_Device ( File : in File_Descriptor ) return Boolean ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Is_A_Terminal_Device
--
-- <DESCRIPTION> This function returns True if the external file associated 
--               with the parameter File is a terminal device.  The function
--               returns False otherwise.
--
-- <EXCEPTIONS>  Bad_File_Descriptor =>
--                 The parameter File is not open.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  function Get_Terminal_Device_Name 
    ( File : in File_Descriptor ) return OSI_String ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Get_Terminal_Device_Name
--
-- <DESCRIPTION> This function returns the OSI_String which represents the 
--               name of the external terminal device associated with the 
--               parameter File.  The function returns an empty OSI_String if 
--               the file associated with the parameter File is not a terminal
--               device.
--
-- <EXCEPTIONS>  Bad_File_Descriptor =>
--                 The parameter File is not open.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

end OSI_Files ;

-- </ADA>
