-- <!DOCTYPE ADA>
-- <ADA>

--------------------------------------------------------------------------------
-- <PROLOGUE>
-- <UNIT>        OSI_Processes
--
-- <DESCRIPTION> This package defines interfaces for executing and passing
--               parameters to processes and reading attributes of the current
--               process.
--
-- <HISTORY>
--
-- <ORIGIN>
-- <AUTHOR>      Gerardo A. Colon
-- <DATE>        11 July 1990
--
-- <COMPANY>     Science Applications International Corporation
-- <ADDRESS>     311 Park Place Boulevard, Suite 360
-- <ADDRESS>     Clearwater, Florida 34619
--
-- <REMARK>      Developed for the STARS program under task S40.
-- </ORIGIN>
--
-- </HISTORY>
--
-- <NOTES>       Dependencies =>
--                 ( Operating_System => None ,
--                   Compiler         => None ,
--                   Device           => None ) ;
--
-- </PROLOGUE>
--------------------------------------------------------------------------------

with OSI_Standard ;

use  OSI_Standard ;

pragma Page ;

--------------------------------------------------------------------------------
-- package OSI_Processes (specification).
--------------------------------------------------------------------------------

package OSI_Processes is

  procedure Invoke_Process ( File_Name      : in OSI_String ;
                             Parameter_List : in OSI_String ) ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Invoke_Process
--
-- <DESCRIPTION> This procedure executes the file identified by File_Name using
--               the process parameters given by Parameter_List.  The procedure
--               determines whether File_Name identifies either an executable 
--               image file or a command script file and executes the file 
--               accordingly.
--
-- <EXCEPTIONS>  Exec_Format_Error                =>
--                 The new process image file is not in proper format.
--               Filename_Syntax_Error            =>
--                 File_Name contains one or more characters not supported
--                 by the implementation for a file name.
--               Filename_Too_Long                =>
--                 The length of the File_Name exceeds it maximum length.
--               Not_Enough_Space                 =>
--                 The process requires more space than the system is able to 
--                 supply.
--               No_Such_File_Or_Directory        =>
--                 File_Name is null, or one or more components of the new
--                 process image file's pathname do not exist.
--               Not_A_Directory                  =>
--                 A component of the path prefix in File_Name is not a 
--                 directory.
--               Permission_Denied                =>
--                 The File_Name identifies a file which is not a regular file, 
--                 or search permission is denied for a directory listed in the 
--                 new process image's file path prefix, or the new process 
--                 image file denies execute permission.
--               Resource_Temporarily_Unavailable =>
--                 The system lacks the necessary resources to create another
--                 process.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  function Parameter_List return String ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Parameter_List
--
-- <DESCRIPTION> This function returns a string containing the parameters of
--               the current process.
--
-- <EXCEPTIONS>  None.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  function Environment_Value_Of ( Name : in OSI_String ) return String ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Environment_Value_Of
--
-- <DESCRIPTION> This function searches the environment for a variable whose
--               name matches the parameter Name.  If such a variable is found,
--               its value is returned as a value of type String.  If the 
--               variable is found but it has no associated value, a null string
--               is returned.  Otherwise an exception is raised.
--
-- <EXCEPTIONS>  Invalid_Argument               =>
--                 The value of Name is a null string or contains '='.
--               Undefined_Environment_Variable =>
--                 There is no name that matches the parameter Name in the 
--                 environment.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

  function Get_Effective_User_Login_Name return String ;
--------------------------------------------------------------------------------
-- <SUBPROGRAM>
-- <UNIT>        Get_Effective_User_Login_Name
--
-- <DESCRIPTION> This function returns the name of the user of the current
--               process.
--
-- <EXCEPTIONS>  None.
--
-- </SUBPROGRAM>
--------------------------------------------------------------------------------

end OSI_Processes ;

-- </ADA>
